@@ -466,19 +466,19 @@ class EmailManager:
                 # character reference
                 try:
                     if text[:3] == "&#x":
-                        return unichr(int(text[3:-1], 16))
+                        return chr(int(text[3:-1], 16))
                     else:
-                        return unichr(int(text[2:-1]))
+                        return chr(int(text[2:-1]))
                 except ValueError:
                     pass
             else:
                 # named entity
                 try:
-                    text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
+                    text = chr(htmlentitydefs.name2codepoint[text[1:-1]])
                 except KeyError:
                     pass
             return text  # leave as is
-        return re.sub("&#?\w+;", fixup, html)
+        return re.sub(r"&#?\w+;", fixup, html)
 
     def decode_quoted_printable(self, html):
         """
