@@ -24,12 +24,14 @@ import os
 import sys
 import time
 import traceback
-from seleniumbase.config import settings
 from selenium.webdriver.common.by import By
 from selenium.webdriver.common.action_chains import ActionChains
 from selenium.webdriver.remote.errorhandler import ElementNotVisibleException
 from selenium.webdriver.remote.errorhandler import NoSuchElementException
 from selenium.webdriver.remote.errorhandler import NoAlertPresentException
+from selenium.webdriver.remote.errorhandler import NoSuchFrameException
+from selenium.webdriver.remote.errorhandler import NoSuchWindowException
+from seleniumbase.config import settings
 
 
 def is_element_present(driver, selector, by=By.CSS_SELECTOR):
@@ -122,8 +124,33 @@ def hover_and_click(driver, hover_selector, click_selector,
     hover.perform()
     for x in range(int(timeout * 10)):
         try:
-            element = driver.find_element(by=click_by,
-                                          value="%s" % click_selector).click()
+            element = driver.find_element(by=click_by, value=click_selector)
+            element.click()
+            return element
+        except Exception:
+            now_ms = time.time() * 1000.0
+            if now_ms >= stop_ms:
+                break
+            time.sleep(0.1)
+    raise NoSuchElementException(
+        "Element {%s} was not present after %s seconds!" %
+        (click_selector, timeout))
+
+
+def hover_element_and_click(driver, element, click_selector,
+                            click_by=By.CSS_SELECTOR,
+                            timeout=settings.SMALL_TIMEOUT):
+    """
+    Similar to hover_and_click(), but assumes top element is already found.
+    """
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    hover = ActionChains(driver).move_to_element(element)
+    hover.perform()
+    for x in range(int(timeout * 10)):
+        try:
+            element = driver.find_element(by=click_by, value=click_selector)
+            element.click()
             return element
         except Exception:
             now_ms = time.time() * 1000.0
@@ -202,14 +229,17 @@ def wait_for_element_visible(driver, selector, by=By.CSS_SELECTOR,
             if now_ms >= stop_ms:
                 break
             time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     if not element and by != By.LINK_TEXT:
         raise ElementNotVisibleException(
-            "Element {%s} was not visible after %s seconds!" % (
-                selector, timeout))
+            "Element {%s} was not visible after %s second%s!" % (
+                selector, timeout, plural))
     if not element and by == By.LINK_TEXT:
         raise ElementNotVisibleException(
-            "Link text {%s} was not visible after %s seconds!" % (
-                selector, timeout))
+            "Link text {%s} was not visible after %s second%s!" % (
+                selector, timeout, plural))
 
 
 def wait_for_text_visible(driver, text, selector, by=By.CSS_SELECTOR,
@@ -246,10 +276,13 @@ def wait_for_text_visible(driver, text, selector, by=By.CSS_SELECTOR,
             if now_ms >= stop_ms:
                 break
             time.sleep(0.1)
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     if not element:
         raise ElementNotVisibleException(
-            "Expected text {%s} for {%s} was not visible after %s seconds!" %
-            (text, selector, timeout))
+            "Expected text {%s} for {%s} was not visible after %s second%s!" %
+            (text, selector, timeout, plural))
 
 
 def wait_for_element_absent(driver, selector, by=By.CSS_SELECTOR,
@@ -276,8 +309,11 @@ def wait_for_element_absent(driver, selector, by=By.CSS_SELECTOR,
             time.sleep(0.1)
         except Exception:
             return True
-    raise Exception("Element {%s} was still present after %s seconds!" %
-                    (selector, timeout))
+    plural = "s"
+    if timeout == 1:
+        plural = ""
+    raise Exception("Element {%s} was still present after %s second%s!" %
+                    (selector, timeout, plural))
 
 
 def wait_for_element_not_visible(driver, selector, by=By.CSS_SELECTOR,
@@ -307,9 +343,12 @@ def wait_for_element_not_visible(driver, selector, by=By.CSS_SELECTOR,
                 return True
         except Exception:
             return True
+    plural = "s"
+    if timeout == 1:
+        plural = ""
     raise Exception(
-        "Element {%s} was still visible after %s seconds!" % (
-            selector, timeout))
+        "Element {%s} was still visible after %s second%s!" % (
+            selector, timeout, plural))
 
 
 def find_visible_elements(driver, selector, by=By.CSS_SELECTOR):
@@ -329,16 +368,25 @@ def save_screenshot(driver, name, folder=None):
     """
     Saves a screenshot to the current directory (or to a subfolder if provided)
     If the folder provided doesn't exist, it will get created.
+    The screenshot will be in PNG format.
     """
+    if "." not in name:
+        name = name + ".png"
     if folder:
         abs_path = os.path.abspath('.')
         file_path = abs_path + "/%s" % folder
         if not os.path.exists(file_path):
             os.makedirs(file_path)
-        screenshot_file = "%s/%s" % (file_path, name)
+        screenshot_path = "%s/%s" % (file_path, name)
     else:
-        screenshot_file = name
-    driver.get_screenshot_as_file(screenshot_file)
+        screenshot_path = name
+    try:
+        element = driver.find_element_by_tag_name('body')
+        element_png = element.screenshot_as_png
+        with open(screenshot_path, "wb") as file:
+            file.write(element_png)
+    except Exception:
+        driver.get_screenshot_as_file(screenshot_path)
 
 
 def _get_last_page(driver):
@@ -377,29 +425,6 @@ def save_test_failure_data(driver, name, browser_type, folder=None):
     failure_data_file.close()
 
 
-def wait_for_ready_state_complete(driver, timeout=settings.EXTREME_TIMEOUT):
-    """
-    The DOM (Document Object Model) has a property called "readyState".
-    When the value of this becomes "complete", page resources are considered
-    fully loaded (although AJAX and other loads might still be happening).
-    This method will wait until document.readyState == "complete".
-    """
-
-    start_ms = time.time() * 1000.0
-    stop_ms = start_ms + (timeout * 1000.0)
-    for x in range(int(timeout * 10)):
-        ready_state = driver.execute_script("return document.readyState")
-        if ready_state == u'complete':
-            return True
-        else:
-            now_ms = time.time() * 1000.0
-            if now_ms >= stop_ms:
-                break
-            time.sleep(0.1)
-    raise Exception(
-        "Page elements never fully loaded after %s seconds!" % timeout)
-
-
 def wait_for_and_accept_alert(driver, timeout=settings.LARGE_TIMEOUT):
     """
     Wait for and accept an alert. Returns the text from the alert.
@@ -450,3 +475,65 @@ def wait_for_and_switch_to_alert(driver, timeout=settings.LARGE_TIMEOUT):
                 break
             time.sleep(0.1)
     raise Exception("Alert was not present after %s seconds!" % timeout)
+
+
+def switch_to_frame(driver, frame, timeout=settings.SMALL_TIMEOUT):
+    """
+    Wait for an iframe to appear, and switch to it. This should be usable
+    as a drop-in replacement for driver.switch_to.frame().
+    @Params
+    driver - the webdriver object (required)
+    frame - the frame element, name, or index
+    timeout - the time to wait for the alert in seconds
+    """
+
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    for x in range(int(timeout * 10)):
+        try:
+            driver.switch_to.frame(frame)
+            return True
+        except NoSuchFrameException:
+            now_ms = time.time() * 1000.0
+            if now_ms >= stop_ms:
+                break
+            time.sleep(0.1)
+    raise Exception("Frame was not present after %s seconds!" % timeout)
+
+
+def switch_to_window(driver, window, timeout=settings.SMALL_TIMEOUT):
+    """
+    Wait for a window to appear, and switch to it. This should be usable
+    as a drop-in replacement for driver.switch_to.window().
+    @Params
+    driver - the webdriver object (required)
+    window - the window index or window handle
+    timeout - the time to wait for the window in seconds
+    """
+
+    start_ms = time.time() * 1000.0
+    stop_ms = start_ms + (timeout * 1000.0)
+    if isinstance(window, int):
+        for x in range(int(timeout * 10)):
+            try:
+                window_handle = driver.window_handles[window]
+                driver.switch_to.window(window_handle)
+                return True
+            except IndexError:
+                now_ms = time.time() * 1000.0
+                if now_ms >= stop_ms:
+                    break
+                time.sleep(0.1)
+        raise Exception("Window was not present after %s seconds!" % timeout)
+    else:
+        window_handle = window
+        for x in range(int(timeout * 10)):
+            try:
+                driver.switch_to.window(window_handle)
+                return True
+            except NoSuchWindowException:
+                now_ms = time.time() * 1000.0
+                if now_ms >= stop_ms:
+                    break
+                time.sleep(0.1)
+        raise Exception("Window was not present after %s seconds!" % timeout)
